/*
  Q Light Controller
  Marquee.js


  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0.txt

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

// Development tool access
var testAlgo;

(
    /**
     * This algorithm produces a marquee lighting effect. 
     */
    function()
    {
        var algo = new Object;
        algo.apiVersion = 2;
        algo.name = "Marquee Sign";
        algo.author = "Based off scripts from Heikki Junnila, Massimo Callegari, and XXX with mods from Carl (Drew) Eisenbeis";
        algo.acceptColors = 2;
        algo.properties = new Array(); 
        algo.inverse = 0;
        algo.properties.push("name:inverse|type:list|display:Illumination Mode|values:Normal,Inversed|write:setInverse|read:getInverse");
        algo.channelmode = 3;
        algo.properties.push("name:channelmode|type:list|display:Channel Mode|values:3,4|write:setChannelmode|read:getChannelmode");
        algo.pattern = 0;
        algo.properties.push("name:pattern|type:list|display:Marquee Pattern|values:Lines,Stripes,Spaced|write:setPattern|read:getPattern");
        algo.cmode = 0;
        algo.properties.push("name:cmode|type:list|display:Color Mode|values:User Input,Random|write:setCmode|read:getCmode");
        algo.orientation = 0;
        algo.properties.push("name:orientation|type:list|display:Orientation|values:Horizontal,Vertical|write:setOrientation|read:getOrientation");
        algo.presetSize = 1;
        algo.properties.push("name:presetSize|type:range|display:Size|values:1,12|write:setSize|read:getSize");
        
        algo.setSize = function(_size)
        {
          algo.presetSize = _size;
          util.initialize();
        }
        
        algo.getSize = function()
        {
          return algo.presetSize;
        }
        
        
        algo.setOrientation = function (_orientation){
        	if (_orientation == "Horizontal") algo.orientation = 0;
        	else if (orientation == "Vertical") algo.orientation = 1;
        	else algo.orientation = 0;
        	util.initialize();
        }
        
      algo.getOrientation = function (){
        	
        	if (algo.orientation == "0") return "Horizontal";
        	else if (algo.orientation == "1") return "Vertical";
        	else return "Horizontal";
        }
        
        algo.setCmode = function (_cmode){
        	if (_cmode == "User Input") algo.cmode = 0;
        	else if (_cmode == "Random") algo.cmode = 1;
        	else algo.cmode = 0;
        	util.initialize();
        }
        
        algo.getCmode = function (){
        	
        	if (algo.cmode == "0") return "User Input";
        	else if (algo.cmode == "1") return "Random";
        	else return "User Input";
        }

        algo.setPattern = function (_pattern){
        	if (_pattern == "Lines") algo.pattern = -1;
        	else if (_pattern == "Stripes") algo.pattern = 0;
        	else if (_pattern == "Spaced") algo.pattern = -2;
        	else algo.pattern = 0;
        	util.initialize();
        }
        
        algo.getPattern = function (){
        	if (algo.pattern == "-1") return "Lines";
        	else if (algo.pattern == "0") return "Stripes";
        	else if (algo.pattern == "-2") return "Spaced";
        	else return "Stripes";
        }
        
        
        algo.setChannelmode = function(_channelmode){
        	if (_channelmode == "3") algo.channelmode = 3;
        	else if (_channelmode == "4") algo.channelmode = 4;
        	else algo.channelmode = 3;
        	util.initialize();
        }
        
        algo.getChannelmode = function (){
        	if (algo.channelmode == "3") return "3";
        	else if (algo.channelmode == "4") return "4";
        	else return "4";
        }
        
        
        algo.setInverse = function(_inverse)
        {
          if (_inverse == "Normal") algo.inverse = 0;
          else if (_inverse == "Inversed") algo.inverse = 1;
          else algo.inverse = 0;
          util.initialize();
        }
        
        algo.getInverse = function()
        {
          if (algo.inverse == 0) return "Normal";
          else if (algo.inverse == 1) return "Inversed";
          else return "Normal";
        }
        
        
        //calculates random rgb values
        function randrgb() { 
        	var r = (Math.random()*256); 
        	var g = (Math.random()*256); 
        	var b = (Math.random()*256); 
        	var randrgb = ( ( r & 0xFF) << 16) + ( ( g & 0xFF) << 8) + ( b & 0xFF ); 
        	return randrgb; 
        	} 
        
      
        
        /**
         * The actual "algorithm" for this RGB script. Produces a map of
         * size($width, $height) each time it is called.
         *
         * @param step The step number that is requested (0 to (algo.rgbMapStepCount - 1))
         * @param rgb Tells the color requested by user in the UI.
         * @return A two-dimensional array[height][width].
         */
        algo.rgbMap = function(width, height, rgb, step)
        {
            var map = new Array(height);
            var i = step;
  
            for (var y = 0; y < height; y++)
            {
                map[y] = new Array();
                for (var x = algo.pattern; x < width; x++)
                {
                  
                	if ((i % algo.channelmode) == 0) {	
                		if (algo.inverse == 0 && algo.cmode == 0) map[y][x] = rgb;
                		else if (algo.inverse == 0 && algo.cmode == 1) map[y][x] = randrgb();
                		else if (algo.inverse == 1) map[y][x] = 0
                	}
                    else {
                    	if (algo.inverse == 0) map[y][x] = 0;
                        else if (algo.inverse == 1 && algo.cmode == 0) map[y][x] = rgb;
                        else if (algo.inverse == 1 && algo.cmode == 1) map[y][x] = randrgb();
                    }
                    i++;
                  
                }
            }

            return map;
        }

        /**
         * Tells RGB Matrix how many steps this algorithm produces with size($width, $height)
         *
         * @param width The width of the map
         * @param height The height of the map
         * @return Number of steps required for a map of size($width, $height)
         */
        algo.rgbMapStepCount = function(width, height)
        {
            // provides for 3 or 4 steps to be returned - simulating a 3 or 4 channel chase controller.
            return algo.channelmode;
        }

        // Development tool access
        testAlgo = algo;

        return algo;
    }
)()
